<?php
namespace Modules\OperadorLogistico\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Modules\OperadorLogistico\Services\ConsolidacaoOLService;

class ConsolidacaoOLController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $dataSourceManager;
    protected $entityManager;
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, EntityManager $entityManager, ConsolidacaoOLService $service){
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
        $this->service = $service;
	}
	
	protected function getProduto($conditions){
	    return $this->crudService->getFilteredDataSet("sup07600_getProdconsolOL", $conditions)->getRows(); 
	}
	
    protected function saveProduto($row){
        $produto = array();
        array_push($produto, $row);
        $this->dataSourceName = "sup07600_getProdconsolOL";
        $this->crudService->save($this->dataSourceName, $produto);
        return $produto;
    }

    public function getConsolidacaoOL(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $resposta = $this->getProduto($conditions);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    protected function getEntrega($conditions){
        return $this->crudService->getFilteredDataSet("sup07600_getEntregaConsOL", $conditions)->getRows();   
    }

    protected function saveEntrega($entrega){
        $this->dataSourceName = "sup07600_getEntregaConsOL";
        $this->crudService->save($this->dataSourceName, $entrega);
    }
    
    public function getEntregaConsOL(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();    
        $resposta = $this->getEntrega($conditions);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    protected function getEstoque($conditions){
        return $this->crudService->getFilteredDataSet("sup07600_getEstoqueConsOL", $conditions)->getRows();   
    }
    
    protected function saveEstoque($estoque){
        $this->dataSourceName = "sup07600_getEstoqueConsOL";
        $this->crudService->save($this->dataSourceName, $estoque);
    }
    
    public function getEstoqueConsOL(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();    
        $resposta = $this->getEstoque($conditions);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function getFilialEntregaConsOL(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();    
        $resposta = $this->crudService->getFilteredDataSet("sup07600_getFilialEntregaConsOL", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function getSolicitacoes(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions(); 
        $resposta = $this->crudService->getFilteredDataSet("sup07600_Solicitacoes", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function deleteConsolidacaoOL(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
    	$this->service->updateItemSolCons($row);
    	$this->service->deleteConsSolOL($row);
    	$this->service->deleteVeriConsol($row);
    }
    
    protected function getProdNeceComp($row){
        return $this->crudService->getFilteredDataSet("sup07600_getProdNeceComp", $this->service->criaFiltroProdNeceComp($row))->getRows();   
    }
    
    /*Método chamado ao finalizar o cálculo da Necessida de OL*/
    public function setEstoqueConsOL(DTO\Request\DataSet $request, DTO\Response $response){
        $produto = $request->getDataSet()->getRows();
        $newProduto = array();
        $newEstoque = array();
       
        foreach ($produto as $rowProduto) {
            if($rowProduto['IDCALESTCONSOL'] == 'S'){
                
                $estoque = $this->getEstoque($this->service->criaFiltroEstoque($rowProduto));
                $entrega = $this->getEntrega($this->service->criaFiltroEstoque($rowProduto));
                $prodnec = $this->getProdNeceComp($rowProduto);
                
                
                if(sizeof($estoque) == 1){
                    foreach ($estoque as $rowEstoque) {
                        if(sizeof($prodnec) > 0){
                            $rowEstoque['QTESTOPRODOL'] = $rowEstoque['QTORIGPRODOL'];
                            $rowEstoque['QTESTOPRODOL'] = 'N';
                        } else {
                            $rowEstoque['IDUTESTCONSOL'] = 'S';
                        }
                        array_push($newEstoque, $rowEstoque);
                    }
                    $resposta = $this->service->setEstoque($newEstoque, $rowProduto);
                    $rowProduto['QTESTUTILCONSOL'] = $resposta[1];
                    $this->saveEstoque($resposta[0]);

                    /*Grava_cdsData*/
                    $newEntrega = $this->service->setEntrega($entrega, $rowProduto['QTESTUTILCONSOL'], $prodnec);
                    $this->saveEntrega($newEntrega);
                }
                
                if(sizeof($prodnec) > 0){
                    $rowProduto['QTESTUTILCONSOL'] = 0;
                }
            }
            array_push($newProduto, $rowProduto);
        }
        $this->saveProduto($rowProduto);
        $response->addDataSet(new DataSet("resposta", $newProduto));
    }
    
    public function setQtdeEstUtl(DTO\Request\Row $request, DTO\Response $response){
        
        $row = $request->getRow();
        $entrega = $entrega = $this->getEntrega($this->service->criaFiltroEstoque($row));
        $newEntrega = $this->service->setEntrega($entrega, $row['QTESTUTILCONSOL'], $this->getProdNeceComp($row));
        
        $row["QTESTUTILCONSOL"] = $this->valueToFloat($row["QTESTUTILCONSOL"]);
        $row["QTANTESTUTIL"] = $this->valueToFloat($row["QTANTESTUTIL"]);

        /*Justificativa*/
        if(strlen($row['DSJUSTCONSOL']) > 0){
            forEach($newEntrega as $rowEntr){
                if($rowEntr['IDENTREGAVIAOL'] != $rowEntr['IDENTREGAVIAOL_OLD']){
                    if($rowEntr['IDENTREGAVIAOL'] == 'S'){
                        $this->service->insertJustificativa($row, $row['DSVIAOL'] . $row['DSJUSTCONSOL']);   
                    } else {
                        $this->service->insertJustificativa($row, $row['DSVIAPONTO'] . $row['DSJUSTCONSOL']);     
                    }
                }
            }
        }
        
        $this->saveEntrega($newEntrega);
        $produto = $this->saveProduto($row);

        /*Justificativa*/
        if(strlen($row['DSJUSTCONSOL']) > 0){
            if($row["QTANTESTUTIL"] != $row["QTESTUTILCONSOL"]){
                $this->service->insertJustificativa($row, $row['DSESTUTIL'] . $row['DSJUSTCONSOL']);   
            }
        }
        
       $response->addDataSet(new DataSet("resposta", $produto));
    }
    
    // Adaptado de 'General.js'
    public function valueToFloat($value) {
		if (is_string($value) && $value) {
			if (strpos($value, ',') === false && strpos($value, '.') === false) {
				return floatval($value);
			} else if (strpos($value, ',') > strpos($value, '.')) {
                $value = str_replace('.', '', $value);
                $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') < strpos($value, '.')) {
			    $value = str_replace(',', '.', $value);
				return floatval($value);
			}
		} else {
			return $value;
		}
    }
    
    public function setIdEntregaviaOL(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        if(sizeof($this->getProdNeceComp($row)) > 0){
            $row['IDENTREGAVIAOL'] = 'S';
            $this->service->updateIdEntregaviaOL($row);
        } else {
            $row["QTESTUTILCONSOL"] = $this->valueToFloat($row["QTESTOCONSOL"]);
            $row["QTANTESTUTIL"] = $this->valueToFloat($row["QTESTOCONSOL"]);
            
            $this->service->updateIdEntregaviaOL($row);
            
            if($row['IDENTREGAVIAOL'] == 'S'){
                $this->service->insertJustificativa($row, 'Via OL - ' . $row['DSJUSTCONSOL']);   
            } else {
                $this->service->insertJustificativa($row, 'Direta - ' . $row['DSJUSTCONSOL']);     
            }
        }
        $newEntrega = array();
        array_push($newEntrega, $row);
        $response->addDataSet(new DataSet("resposta", $newEntrega));
    }
    
    public function setConsideraEstoque(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $newEstoque = array();
        $produto = $this->getProduto($this->service->criaFiltroProdConsOL($row));
        foreach ($produto as $rowProduto){
            $prodnec = $this->getProdNeceComp($rowProduto);
            if(sizeof($prodnec) > 0){
                $row['QTESTOPRODOL'] = $row['QTORIGPRODOL'];
                $row['QTESTOPRODOL'] = 'N';
            }
            array_push($newEstoque, $row);
            $resposta = $this->service->setEstoque($newEstoque, $rowProduto);
            $this->saveEstoque($resposta[0]);
            $rowProduto['QTESTUTILCONSOL'] = $resposta[1];
            
            /*Grava_cdsData*/
            $entrega = $this->getEntrega($this->service->criaFiltroEstoque($rowProduto));
            $newEntrega = $this->service->setEntrega($entrega, $rowProduto['QTESTUTILCONSOL'], $prodnec);
            $this->saveEntrega($newEntrega);
            
            /*Justificativa*/
            if(strlen($row['DSJUSTCONSOL']) > 0){
                forEach($newEntrega as $rowEntr){
                    if($rowEntr['IDENTREGAVIAOL'] != $rowEntr['IDENTREGAVIAOL_OLD']){
                        if($rowEntr['IDENTREGAVIAOL'] == 'S'){
                            $this->service->insertJustificativa($row, $row['DSVIAOL'] . $row['DSJUSTCONSOL']);   
                        } else {
                            $this->service->insertJustificativa($row, $row['DSVIAPONTO'] . $row['DSJUSTCONSOL']);     
                        }
                    }
                }
            }

            if(sizeof($prodnec) > 0){
                $rowProduto['QTESTUTILCONSOL'] = 0;
            }
            
            /*Justificativa*/
            if(strlen($row['DSJUSTCONSOL']) > 0){
                if($row["QTANTESTUTIL"] != $rowProduto["QTESTUTILCONSOL"]){
                    $row["QTESTUTILCONSOL"] = $rowProduto["QTESTUTILCONSOL"];
                    $this->service->insertJustificativa($row, $row['DSESTUTIL'] . $row['DSJUSTCONSOL']);   
                }
            }
            
            $this->saveProduto($rowProduto);

            $resposta[0] = $this->getEntrega($this->service->criaFiltroEstoque($rowProduto));
            $resposta[1] = $this->getEstoque($this->service->criaFiltroEstoque($rowProduto));
        }
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function geraSolicitacaoOL(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $resposta = $this->service->procedureGeraSolicitacaoOL($row[0]);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function getTotalSolicitado(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions(); 
        $resposta = $this->crudService->getFilteredDataSet("sup07600_getQtdeSolicitada", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function incluirProduto(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $resposta = $this->service->procedureNecessidadeOL($row[0]);
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function getInsconsConsoOL(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions(); 
        $resposta = $this->crudService->getFilteredDataSet("sup07600_getInsconsConsOL", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
}